import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.hadoop.io.Writable;

/**
 * This class handles in- and output issues for a set of bindings of variables to their correponding values.
 */
public class BindingsWritable implements Writable, Iterable<Map.Entry<String, String>> {
	
	/**
	 * The map in which the variable/value-pairs are hold in main memory
	 */
	private Map<String, String> bindings = new HashMap<String, String>();

	/**
	 * Returns the bound value of a given variable
	 * @param variable the variable the bound value of which will be returned
	 * @return the bound value of the given variable, or null if the variable is not bound
	 */
	public String get(String variable){
		return this.bindings.get(variable);
	}
	
	/**
	 * This methods binds a given variable to the given value
	 * @param variable the variable to bind
	 * @param value the value to which the variable is bound
	 */
	public void put(String variable, String value){
		this.bindings.put(variable, value);
	}
	
	/**
	 * Returns the number of bound variables
	 * @return number of bound variables
	 */
	public int size(){
		return this.bindings.size();
	}
	
	/**
	 * Returns the set of variables, which are bound
	 * @return the set of variables, which are bound
	 */
	public Set<String> getVariables(){
		return this.bindings.keySet();
	}
	
	/**
	 * Returns the set of variable/value-pairs
	 * @return the set of variable/value-pairs
	 */
	public Set<Map.Entry<String, String>> getBindings(){
		return this.bindings.entrySet();
	}
	
	public Iterator<Entry<String, String>> iterator() {
		return this.getBindings().iterator();
	}
	
	/**
	 * Provides a pretty-printing of the variable/value-pairs
	 * @return a pretty-printing of the variable/value-pairs
	 */
	public String toString() {
		StringBuilder resultBuilder = new StringBuilder();
		resultBuilder.append("{");
		boolean firstTime = true;
		for(Map.Entry<String, String> entry: this.bindings.entrySet()){
			if(firstTime){
				firstTime = false;
			} else {
				resultBuilder.append(", ");
			}
			resultBuilder.append(entry.getKey());
			resultBuilder.append(" = ");
			resultBuilder.append(entry.getValue());
		}
		resultBuilder.append("}");
		return resultBuilder.toString();
	}
	
	/**
	 * Creates a clone of this object
	 * @return a clone of this object
	 */
	public BindingsWritable clone(){
		BindingsWritable clone = new BindingsWritable();
		for(Entry<String, String> entry: this){
			clone.put(entry.getKey(), entry.getValue());
		}
		return clone;
	}
	
	/**
	 * Removes all variable/value-pairs
	 */
	public void clear() {
		this.bindings.clear();
	}
	
	/**
	 * For reading the fields of this object
	 * @param in the data input from which is read
	 */
	public void readFields(DataInput in) throws IOException {
		this.clear();
		int size = in.readInt();
		for(int i=0; i<size; i++){
			String variable = in.readUTF();
			String value = in.readUTF();
			this.bindings.put(variable, value);
		}
	}

	/**
	 * For writing the field of this object
	 * @param out the data out to which is written
	 */
	public void write(DataOutput out) throws IOException {
		out.writeInt(this.bindings.size());
		for(Map.Entry<String, String> entry: bindings.entrySet()){
			out.writeUTF(entry.getKey());
			out.writeUTF(entry.getValue());
		}
	}
	
	/**
	 * A static method for creating a new BindingsWritable object and reading in its fields
	 * @param in the data input from which is read
	 * @return the newly created BindingsWritable object
	 * @throws IOException
	 */
    public static BindingsWritable read(DataInput in) throws IOException {
    	BindingsWritable w = new BindingsWritable();
        w.readFields(in);
        return w;
      }
}
