import java.io.IOException;
 
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;

/**
 * The word count example, which counts all words (read from a comma-separated-values (csv) file)
 */
public class WordCount {
 
	/**
	 * The main method of the word count example initializing and submitting the WordCount job to hadoop.
	 * @param args the input and output path
	 * @throws IOException
	 * @throws InterruptedException
	 * @throws ClassNotFoundException
	 */
    public static void main(String[] args) throws IOException, InterruptedException, ClassNotFoundException {
 
        Path inputPath = new Path(args[0]);
        Path outputDir = new Path(args[1]);
 
        // Create configuration
        Configuration conf = new Configuration(true);
 
        // Create job
        Job job = new Job(conf, "WordCount");
        job.setJarByClass(WordCountMapper.class);
 
        // Setup MapReduce
        job.setMapperClass(WordCountMapper.class);
        job.setReducerClass(WordCountReducer.class);
        job.setNumReduceTasks(1);
 
        // Specify key / value
        job.setOutputKeyClass(Text.class);
        job.setOutputValueClass(IntWritable.class);
 
        // Input
        FileInputFormat.addInputPath(job, inputPath);
        job.setInputFormatClass(TextInputFormat.class);
 
        // Output
        FileOutputFormat.setOutputPath(job, outputDir);
        job.setOutputFormatClass(TextOutputFormat.class);
 
        // Delete output if exists
        FileSystem hdfs = FileSystem.get(conf);
        if (hdfs.exists(outputDir))
            hdfs.delete(outputDir, true);
 
        // Execute job
        int code = job.waitForCompletion(true) ? 0 : 1;
        System.exit(code);
 
    }
    
    /**
     * The class for the hadoop mapper splitting the input into words and mapping them to word,1
     */
    public static class WordCountMapper extends Mapper<Object, Text, Text, IntWritable> {

    	private final IntWritable ONE = new IntWritable(1); // reusable object to save resources
    	private Text word = new Text(); // reusable object to save resources

    	/**
    	 * The main map method mapping each word w to the key w with value 1
    	 */
    	public void map(Object key, Text value, Context context)
    			throws IOException, InterruptedException {

    		// splitting the input according to the comma (We assume the words are separated by comma)
    		String[] csv = value.toString().split(",");
    		for (String str : csv) {
    			// for each word w write w,1 to the context (for the reduce phase) 
    			word.set(str);
    			context.write(word, ONE);
    		}
    	}
    }

    /**
     * Reducer class to count the words, which were previously mapped to the words themselves with value 1
     */
    public static class WordCountReducer extends Reducer<Text, IntWritable, Text, IntWritable> {

    	/**
    	 * The reduce method, which just iterates through all occurrences of the same word, counts them and writes the word with the counted number
    	 */
    	public void reduce(Text text, Iterable<IntWritable> values, Context context)
    			throws IOException, InterruptedException {
    		int sum = 0;
			// count all occurrences of the current word
    		for (IntWritable value : values) {
    			sum += value.get();
    		}
    		// write the result: the word plus the count of all its occurrences...
    		context.write(text, new IntWritable(sum));
    	}
    }
}